@@ -9,8 +9,10 @@
 ## EXTRACT MODEL ###
 setMethod("semPlotModel_S4",signature("lavaan"),function(object){
 
-  if (class(object)!="lavaan") stop("Input must me a 'lavaan' object")
+  if (is(object,"blavaan")) class(object) <- 'lavaan'
+  if (!is(object,"lavaan")) stop("Input must me a 'lavaan' object")
 
+  
   # Extract parameter estimates:
   pars <- parameterEstimates(object,standardized=TRUE)
   list <- inspect(object,"list")
@@ -76,24 +78,33 @@ setMethod("semPlotModel_S4",signature("lavaan"),function(object){
     exogenous = NA,
     stringsAsFactors=FALSE)
     
-  if (!is.null(object@SampleStats@res.cov[[1]])){
-    semModel@ObsCovs <- object@SampleStats@res.cov    
-  } else {
-    semModel@ObsCovs <- object@SampleStats@cov
-  }
-
-  names(semModel@ObsCovs) <- object@Data@group.label
-  for (i in 1:length(semModel@ObsCovs))
-  {
-    rownames(semModel@ObsCovs[[i]]) <- colnames(semModel@ObsCovs[[i]]) <- object@Data@ov.names[[i]]
-  }
+  # res.cov <- lavTech(object, "sampstat")$res.cov
+  # lavTech(object, "sampstat")$cov
+  # if (!is.null(res.cov) && !length(res.cov) == 0){
+      # if (!is.null(res.cov[[1]])){
+      #   semModel@ObsCovs <- object@SampleStats@res.cov    
+      # } else {
+      #   semModel@ObsCovs <- object@SampleStats@cov
+      # }    
+  # } else {
+  #   semModel@ObsCovs <- list(matrix(NA,
+  #          length(varNames),length(varNames)))
+  # } 
   
-  semModel@ImpCovs <- object@Fit@Sigma.hat
-  names(semModel@ImpCovs) <- object@Data@group.label
-  for (i in 1:length(semModel@ImpCovs))
-  {
-    rownames(semModel@ImpCovs[[i]]) <- colnames(semModel@ImpCovs[[i]]) <- object@Data@ov.names[[i]]
+  # Use add.labels=TRUE so lavTech returns named matrices (handles multigroup with different vars)
+  if (lavInspect(object, "options")$conditional.x){
+    semModel@ObsCovs <- lapply(lavTech(object, "sampstat", add.labels = TRUE),"[[","res.cov")
+  } else {
+    semModel@ObsCovs <- lapply(lavTech(object, "sampstat", add.labels = TRUE),"[[","cov")
   }
+  names(semModel@ObsCovs) <- lavInspect(object, "group.label")
+
+  if (lavInspect(object, "options")$conditional.x){
+    semModel@ImpCovs <- lapply(lavTech(object, "implied", add.labels = TRUE), "[[", "res.cov")
+  } else {
+    semModel@ImpCovs <- lapply(lavTech(object, "implied", add.labels = TRUE), "[[", "cov")
+  }
+  names(semModel@ImpCovs) <- lavInspect(object, "group.label") # object@Data@group.label
   
   semModel@Computed <- TRUE
   
